@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(rabbit_amqp1_0_reader).
@@ -95,15 +95,7 @@ server_properties() ->
     %% The atom doesn't match anything, it's just "not 0-9-1".
     Raw = lists:keydelete(
           <<"capabilities">>, 1, rabbit_reader:server_properties(amqp_1_0)),
-    {map, [{{symbol, binary_to_list(K)}, {utf8, V}}
-           || {K, longstr, V}  <- Raw]}.
-
-%% TODO copypasta from rabbit_federation_util:local_nodename_implicit/0. Unify.
-container_id() ->
-    {ID, _} = rabbit_nodes:parts(node()),
-    {ok, Host} = inet:gethostname(),
-    {ok, #hostent{h_name = FQDN}} = inet:gethostbyname(Host),
-    {utf8, list_to_binary(atom_to_list(rabbit_nodes:make({ID, FQDN})))}.
+    {map, [{{symbol, K}, {utf8, V}} || {K, longstr, V}  <- Raw]}.
 
 %%--------------------------------------------------------------------------
 
@@ -388,7 +380,8 @@ handle_1_0_connection_frame(#'v1_0.open'{ max_frame_size = ClientFrameMax,
                                [FrameMax, ?FRAME_1_0_MIN_SIZE]);
            true ->
                 {ok, Collector} =
-                    rabbit_connection_helper_sup:start_queue_collector(HelperSupPid),
+                    rabbit_connection_helper_sup:start_queue_collector(
+                      HelperSupPid, <<"AMQP 1.0">>), %% TODO describe the connection
                 SendFun =
                     fun() ->
                             Frame =
@@ -422,7 +415,7 @@ handle_1_0_connection_frame(#'v1_0.open'{ max_frame_size = ClientFrameMax,
            #'v1_0.open'{channel_max    = ClientChannelMax,
                         max_frame_size = ClientFrameMax,
                         idle_time_out  = {uint, HeartbeatSec * 1000},
-                        container_id   = container_id(),
+                        container_id   = {utf8, rabbit_nodes:cluster_name()},
                         properties     = server_properties()}),
     Conserve = rabbit_alarm:register(self(), {?MODULE, conserve_resources, []}),
     control_throttle(
@@ -472,7 +465,7 @@ handle_1_0_session_frame(Channel, Frame, State) ->
     end.
 
 %% TODO: write a proper ANONYMOUS plugin and unify with STOMP
-handle_1_0_sasl_frame(#'v1_0.sasl_init'{mechanism = {symbol, "ANONYMOUS"},
+handle_1_0_sasl_frame(#'v1_0.sasl_init'{mechanism = {symbol, <<"ANONYMOUS">>},
                                         hostname = _Hostname},
                       State = #v1{connection_state = starting,
                                   sock             = Sock}) ->
@@ -494,7 +487,7 @@ handle_1_0_sasl_frame(#'v1_0.sasl_init'{mechanism        = {symbol, Mechanism},
                       State0 = #v1{connection_state = starting,
                                    connection       = Connection,
                                    sock             = Sock}) ->
-    AuthMechanism = auth_mechanism_to_module(list_to_binary(Mechanism), Sock),
+    AuthMechanism = auth_mechanism_to_module(Mechanism, Sock),
     State = State0#v1{connection       =
                           Connection#connection{
                             auth_mechanism    = {Mechanism, AuthMechanism},
@@ -655,7 +648,14 @@ auth_phase_1_0(Response,
             ok = send_on_channel0(Sock, Secure, rabbit_amqp1_0_sasl),
             State#v1{connection = Connection =
                          #connection{auth_state = AuthState1}};
-        {ok, User} ->
+        {ok, User = #user{username = Username}} ->
+            case rabbit_access_control:check_user_loopback(Username, Sock) of
+                ok          -> ok;
+                not_allowed -> protocol_error(
+                                 ?V_1_0_AMQP_ERROR_UNAUTHORIZED_ACCESS,
+                                 "user '~s' can only connect via localhost",
+                                 [Username])
+            end,
             Outcome = #'v1_0.sasl_outcome'{code = {ubyte, 0}},
             ok = send_on_channel0(Sock, Outcome, rabbit_amqp1_0_sasl),
             switch_callback(
