@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(rabbit_auth_backend_ldap_app).
@@ -25,17 +25,28 @@
 
 start(_Type, _StartArgs) ->
     {ok, Backends} = application:get_env(rabbit, auth_backends),
-    case lists:member(rabbit_auth_backend_ldap, Backends) of
+    case configured(rabbit_auth_backend_ldap, Backends) of
         true  -> ok;
         false -> rabbit_log:warning(
                    "LDAP plugin loaded, but rabbit_auth_backend_ldap is not "
                    "in the list of auth_backends. LDAP auth will not work.~n")
     end,
+    {ok, SSL} = application:get_env(rabbitmq_auth_backend_ldap, use_ssl),
+    case SSL of
+        true  -> rabbit_networking:ensure_ssl();
+        false -> ok
+    end,
     supervisor:start_link({local, ?MODULE}, ?MODULE, []).
 
 stop(_State) ->
     ok.
 
+configured(_M, [])        -> false;
+configured(M,  [M    |_]) -> true;
+configured(M,  [{M,_}|_]) -> true;
+configured(M,  [{_,M}|_]) -> true;
+configured(M,  [_    |T]) -> configured(M, T).
+
 %%----------------------------------------------------------------------------
 
 init([]) -> {ok, {{one_for_one, 3, 10}, []}}.
