@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 %% @private
@@ -21,7 +21,7 @@
 
 -behaviour(supervisor2).
 
--export([start_link/0, start_infrastructure/3, type_module/1]).
+-export([start_link/0, start_infrastructure_fun/3, type_module/1]).
 
 -export([init/1]).
 
@@ -32,51 +32,52 @@
 start_link() ->
     supervisor2:start_link(?MODULE, []).
 
-start_infrastructure(Sup, Conn, Type) ->
-    {ok, ChMgr} = start_channels_manager(Sup, Conn, Type),
-    {ok, start_infrastructure_fun(Sup, Conn, ChMgr, Type), ChMgr}.
-
 type_module(#amqp_params_direct{})  -> {direct, amqp_direct_connection};
 type_module(#amqp_params_network{}) -> {network, amqp_network_connection}.
 
 %%---------------------------------------------------------------------------
 
-start_channels_manager(Sup, Conn, Type) ->
+start_channels_manager(Sup, Conn, ConnName, Type) ->
     {ok, ChSupSup} = supervisor2:start_child(
                        Sup,
                        {channel_sup_sup, {amqp_channel_sup_sup, start_link,
-                                          [Type, Conn]},
+                                          [Type, Conn, ConnName]},
                         intrinsic, infinity, supervisor,
                         [amqp_channel_sup_sup]}),
     {ok, _} = supervisor2:start_child(
                 Sup,
                 {channels_manager, {amqp_channels_manager, start_link,
-                                    [Conn, ChSupSup]},
+                                    [Conn, ConnName, ChSupSup]},
                  transient, ?MAX_WAIT, worker, [amqp_channels_manager]}).
 
-start_infrastructure_fun(Sup, Conn, ChMgr, network) ->
-    fun (Sock) ->
+start_infrastructure_fun(Sup, Conn, network) ->
+    fun (Sock, ConnName) ->
+            {ok, ChMgr} = start_channels_manager(Sup, Conn, ConnName, network),
             {ok, AState} = rabbit_command_assembler:init(?PROTOCOL),
             {ok, Writer} =
                 supervisor2:start_child(
                   Sup,
-                  {writer, {rabbit_writer, start_link,
-                            [Sock, 0, ?FRAME_MIN_SIZE, ?PROTOCOL, Conn]},
+                  {writer,
+                   {rabbit_writer, start_link,
+                    [Sock, 0, ?FRAME_MIN_SIZE, ?PROTOCOL, Conn, ConnName]},
                    transient, ?MAX_WAIT, worker, [rabbit_writer]}),
             {ok, _Reader} =
                 supervisor2:start_child(
                   Sup,
                   {main_reader, {amqp_main_reader, start_link,
-                                 [Sock, Conn, ChMgr, AState]},
+                                 [Sock, Conn, ChMgr, AState, ConnName]},
                    transient, ?MAX_WAIT, worker, [amqp_main_reader]}),
-            {ok, Writer}
+            {ok, ChMgr, Writer}
     end;
-start_infrastructure_fun(Sup, _Conn, _ChMgr, direct) ->
-    fun () ->
-            supervisor2:start_child(
-              Sup,
-              {collector, {rabbit_queue_collector, start_link, []},
-               transient, ?MAX_WAIT, worker, [rabbit_queue_collector]})
+start_infrastructure_fun(Sup, Conn, direct) ->
+    fun (ConnName) ->
+            {ok, ChMgr} = start_channels_manager(Sup, Conn, ConnName, direct),
+            {ok, Collector} =
+                supervisor2:start_child(
+                  Sup,
+                  {collector, {rabbit_queue_collector, start_link, [ConnName]},
+                   transient, ?MAX_WAIT, worker, [rabbit_queue_collector]}),
+            {ok, ChMgr, Collector}
     end.
 
 %%---------------------------------------------------------------------------
