@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 %% @private
@@ -21,7 +21,7 @@
 
 -behaviour(gen_server).
 
--export([start_link/4]).
+-export([start_link/5]).
 -export([init/1, terminate/2, code_change/3, handle_call/3, handle_cast/2,
          handle_info/2]).
 
@@ -36,14 +36,16 @@
 %% Interface
 %%---------------------------------------------------------------------------
 
-start_link(Sock, Connection, ChMgr, AState) ->
-    gen_server:start_link(?MODULE, [Sock, Connection, ChMgr, AState], []).
+start_link(Sock, Connection, ChMgr, AState, ConnName) ->
+    gen_server:start_link(
+      ?MODULE, [Sock, Connection, ConnName, ChMgr, AState], []).
 
 %%---------------------------------------------------------------------------
 %% gen_server callbacks
 %%---------------------------------------------------------------------------
 
-init([Sock, Connection, ChMgr, AState]) ->
+init([Sock, Connection, ConnName, ChMgr, AState]) ->
+    ?store_proc_name(ConnName),
     State = #state{sock             = Sock,
                    connection       = Connection,
                    channels_manager = ChMgr,
