@@ -11,19 +11,38 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(amqp_uri).
 
 -include("amqp_client.hrl").
 
--export([parse/1, parse/2]).
+-export([parse/1, parse/2, remove_credentials/1]).
 
 %%---------------------------------------------------------------------------
 %% AMQP URI Parsing
 %%---------------------------------------------------------------------------
 
+%% Reformat a URI to remove authentication secrets from it (before we
+%% log it or display it anywhere).
+remove_credentials(URI) ->
+    Props = uri_parser:parse(URI,
+                             [{host, undefined}, {path, undefined},
+                              {port, undefined}, {'query', []}]),
+    PortPart = case proplists:get_value(port, Props) of
+                   undefined -> "";
+                   Port      -> rabbit_misc:format(":~B", [Port])
+               end,
+    PGet = fun(K, P) -> case proplists:get_value(K, P) of
+                            undefined -> "";
+                            R         -> R
+                        end
+           end,
+    rabbit_misc:format(
+      "~s://~s~s~s", [proplists:get_value(scheme, Props), PGet(host, Props),
+                      PortPart,                           PGet(path, Props)]).
+
 %% @spec (Uri) -> {ok, #amqp_params_network{} | #amqp_params_direct{}} |
 %%                {error, {Info, Uri}}
 %% where
@@ -46,15 +65,7 @@
 parse(Uri) -> parse(Uri, <<"/">>).
 
 parse(Uri, DefaultVHost) ->
-    try case parse1(Uri, DefaultVHost) of
-            {ok, #amqp_params_network{host         = undefined,
-                                      username     = User,
-                                      virtual_host = Vhost}} ->
-                return({ok, #amqp_params_direct{username     = User,
-                                                virtual_host = Vhost}});
-            {ok, Params} ->
-                return({ok, Params})
-        end
+    try return(parse1(Uri, DefaultVHost))
     catch throw:Err -> {error, {Err, Uri}};
           error:Err -> {error, {Err, Uri}}
     end.
@@ -106,19 +117,38 @@ build_broker(ParsedUri, DefaultVHost) ->
                              end
             end,
     UserInfo = proplists:get_value(userinfo, ParsedUri),
-    Ps = #amqp_params_network{host            = unescape_string(Host),
-                              port            = Port,
-                              virtual_host    = VHost,
-                              auth_mechanisms = mechanisms(ParsedUri)},
+    set_user_info(case unescape_string(Host) of
+                      undefined -> #amqp_params_direct{virtual_host = VHost};
+                      Host1     -> Mech = mechanisms(ParsedUri),
+                                   #amqp_params_network{host            = Host1,
+                                                        port            = Port,
+                                                        virtual_host    = VHost,
+                                                        auth_mechanisms = Mech}
+                  end, UserInfo).
+
+set_user_info(Ps, UserInfo) ->
     case UserInfo of
-        [U, P | _] -> Ps#amqp_params_network{
-                        username = list_to_binary(unescape_string(U)),
-                        password = list_to_binary(unescape_string(P))};
-        [U | _]    -> Ps#amqp_params_network{
-                        username = list_to_binary(unescape_string(U))};
-        _          -> Ps
+        [U, P | _] -> set([{username, list_to_binary(unescape_string(U))},
+                           {password, list_to_binary(unescape_string(P))}], Ps);
+
+        [U]        -> set([{username, list_to_binary(unescape_string(U))}], Ps);
+        []         -> Ps
     end.
 
+set(KVs, Ps = #amqp_params_direct{}) ->
+    set(KVs, Ps, record_info(fields, amqp_params_direct));
+set(KVs, Ps = #amqp_params_network{}) ->
+    set(KVs, Ps, record_info(fields, amqp_params_network)).
+
+set(KVs, Ps, Fields) ->
+    {Ps1, _Ix} = lists:foldl(fun (Field, {PsN, Ix}) ->
+                                     {case lists:keyfind(Field, 1, KVs) of
+                                          false  -> PsN;
+                                          {_, V} -> setelement(Ix, PsN, V)
+                                      end, Ix + 1}
+                             end, {Ps, 2}, Fields),
+    Ps1.
+
 build_ssl_broker(ParsedUri, DefaultVHost) ->
     Params = build_broker(ParsedUri, DefaultVHost),
     Query = proplists:get_value('query', ParsedUri),
@@ -144,6 +174,8 @@ build_ssl_broker(ParsedUri, DefaultVHost) ->
           []),
     Params#amqp_params_network{ssl_options = SSLOptions}.
 
+broker_add_query(Params = #amqp_params_direct{}, Uri) ->
+    broker_add_query(Params, Uri, record_info(fields, amqp_params_direct));
 broker_add_query(Params = #amqp_params_network{}, Uri) ->
     broker_add_query(Params, Uri, record_info(fields, amqp_params_network)).
 
