@@ -11,7 +11,7 @@
 %%  The Original Code is RabbitMQ.
 %%
 %%  The Initial Developer of the Original Code is GoPivotal, Inc.
-%%  Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%%  Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(rabbit_federation_mgmt).
@@ -72,16 +72,7 @@ status(Node, Chs) ->
         Status                          -> [format(Node, I, Chs) || I <- Status]
     end.
 
-format(Node, Info0, Chs) ->
-    Info1 = proplists:delete(status, Info0),
-    Info = case pget(status, Info0) of
-               {running, Name}   -> [{status,           running},
-                                     {local_connection, Name} | Info1];
-               {Status, E}       -> Fmted = rabbit_mgmt_format:print("~p", [E]),
-                                    [{status, Status},
-                                     {error,  Fmted} | Info1];
-               S when is_atom(S) -> [{status, S} | Info1]
-           end,
+format(Node, Info, Chs) ->
     LocalCh = case rabbit_mgmt_format:strip_pids(
                      [Ch || Ch <- Chs,
                             pget(name, pget(connection_details, Ch))
@@ -97,6 +88,8 @@ format_info(Items) ->
 format_item({timestamp, {{Y, M, D}, {H, Min, S}}}) ->
     {timestamp, print("~w-~2.2.0w-~2.2.0w ~w:~2.2.0w:~2.2.0w",
                       [Y, M, D, H, Min, S])};
+format_item({error, E}) ->
+    {error, rabbit_mgmt_format:print("~p", [E])};
 format_item(I) ->
     I.
 
