@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ Federation.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(rabbit_federation_link_sup).
@@ -58,13 +58,13 @@ adjust(Sup, XorQ, {clear_upstream, UpstreamName}) ->
     [stop(Sup, Upstream, XorQ) || Upstream <- children(Sup, UpstreamName)];
 
 %% TODO handle changes of upstream sets minimally (bug 24853)
-adjust(Sup, X = #exchange{name = XName}, {upstream_set, Set}) ->
+adjust(Sup, X = #exchange{name = XName}, {upstream_set, _Set}) ->
+    adjust(Sup, X, everything),
     case rabbit_federation_upstream:federate(X) of
         false -> ok;
         true  -> ok = rabbit_federation_db:prune_scratch(
                         XName, rabbit_federation_upstream:for(X))
-    end,
-    adjust(Sup, X, everything);
+    end;
 adjust(Sup, Q = #amqqueue{}, {upstream_set, _}) ->
     adjust(Sup, Q, everything);
 adjust(Sup, XorQ, {clear_upstream_set, _}) ->
@@ -91,9 +91,9 @@ children(Sup, UpstreamName) ->
 %%----------------------------------------------------------------------------
 
 init(XorQ) ->
-    %% 1, 1 so that the supervisor can give up and get into waiting
-    %% for the reconnect_delay quickly.
-    {ok, {{one_for_one, 1, 1}, specs(XorQ)}}.
+    %% 1, ?MAX_WAIT so that we always give up after one fast retry and get
+    %% into the reconnect delay.
+    {ok, {{one_for_one, 1, ?MAX_WAIT}, specs(XorQ)}}.
 
 specs(XorQ) ->
     [spec(Upstream, XorQ) || Upstream <- rabbit_federation_upstream:for(XorQ)].
