@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(rabbit_federation_parameters).
@@ -20,8 +20,8 @@
 
 -include_lib("rabbit_common/include/rabbit.hrl").
 
--export([validate/4, notify/4, notify_clear/3]).
--export([register/0, validate_policy/1]).
+-export([validate/5, notify/4, notify_clear/3]).
+-export([register/0, validate_policy/1, adjust/1]).
 
 -rabbit_boot_step({?MODULE,
                    [{description, "federation parameters"},
@@ -38,50 +38,32 @@ register() ->
                           {policy_validator,  <<"federation-upstream-set">>}]],
     ok.
 
-validate(_VHost, <<"federation-upstream-set">>, Name, Term) ->
+validate(_VHost, <<"federation-upstream-set">>, Name, Term, _User) ->
     [rabbit_parameter_validation:proplist(
        Name,
        [{<<"upstream">>, fun rabbit_parameter_validation:binary/2, mandatory} |
         shared_validation()], Upstream)
      || Upstream <- Term];
 
-validate(_VHost, <<"federation-upstream">>, Name, Term) ->
+validate(_VHost, <<"federation-upstream">>, Name, Term, _User) ->
     rabbit_parameter_validation:proplist(
       Name, [{<<"uri">>, fun validate_uri/2, mandatory} |
             shared_validation()], Term);
 
-validate(_VHost, <<"federation">>, <<"local-nodename">>, Term) ->
-    rabbit_parameter_validation:binary(<<"local-nodename">>, Term);
-
-validate(_VHost, <<"federation">>, <<"local-username">>, Term) ->
-    rabbit_parameter_validation:binary(<<"local-username">>, Term);
-
-validate(_VHost, _Component, Name, _Term) ->
+validate(_VHost, _Component, Name, _Term, _User) ->
     {error, "name not recognised: ~p", [Name]}.
 
 notify(_VHost, <<"federation-upstream-set">>, Name, _Term) ->
     adjust({upstream_set, Name});
 
 notify(_VHost, <<"federation-upstream">>, Name, _Term) ->
-    adjust({upstream, Name});
-
-notify(_VHost, <<"federation">>, <<"local-nodename">>, _Term) ->
-    adjust(everything);
-
-notify(_VHost, <<"federation">>, <<"local-username">>, _Term) ->
-    adjust(everything).
+    adjust({upstream, Name}).
 
 notify_clear(_VHost, <<"federation-upstream-set">>, Name) ->
     adjust({clear_upstream_set, Name});
 
 notify_clear(_VHost, <<"federation-upstream">>, Name) ->
-    adjust({clear_upstream, Name});
-
-notify_clear(_VHost, <<"federation">>, <<"local-nodename">>) ->
-    adjust(everything);
-
-notify_clear(_VHost, <<"federation">>, <<"local-username">>) ->
-    adjust(everything).
+    adjust({clear_upstream, Name}).
 
 adjust(Thing) ->
     rabbit_federation_exchange_link_sup_sup:adjust(Thing),
