@@ -32,11 +32,19 @@ HELP = {
     'queue-persistent':
       'Number of messages in the queue which are persistent. These messages will be on disc but may also be available in memory. Note that if a message is published as persistent but routed to a transient queue it is not considered persistent by that queue, so transient queues will always report 0 persistent messages.',
 
+    'queue-consumer-utilisation':
+      'Fraction of the time that the queue is able to immediately deliver messages to consumers. If this number is less than 100% you may be able to deliver messages faster if: \
+        <ul> \
+          <li>There were more consumers or</li> \
+          <li>The consumers were faster or</li> \
+          <li>The consumers had a higher prefetch count</li> \
+        </ul>',
+
     'internal-users-only':
       'Only users within the internal RabbitMQ database are shown here. Other users (e.g. those authenticated over LDAP) will not appear.',
 
     'export-definitions':
-    'The definitions consist of users, virtual hosts, permissions, parameters, exchanges, queues and bindings. They do not include the contents of queues. Exclusive queues will not be exported.',
+    'The definitions consist of users, virtual hosts, permissions, parameters, exchanges, queues and bindings. They do not include the contents of queues or the cluster name. Exclusive queues will not be exported.',
 
     'import-definitions':
       'The definitions that are imported will be merged with the current definitions. If an error occurs during import, any changes made will not be rolled back.',
@@ -54,7 +62,18 @@ HELP = {
         <dd>Channel will send streaming publish confirmations.</dd> \
         <dt><acronym title="Transactional">T</acronym> &ndash; <a target="_blank" href="http://www.rabbitmq.com/amqp-0-9-1-reference.html#class.tx">transactional</a></dt> \
         <dd>Channel is transactional.</dd> \
-      <dl>',
+      </dl>',
+
+    'channel-prefetch':
+      'Channel prefetch counts. \
+       <p> \
+         Each channel can have two prefetch counts: A per-consumer count, which \
+         will limit each new consumer created on the channel, and a global \
+         count, which is shared between all consumers on the channel.\
+       </p> \
+       <p> \
+         This column shows one, the other, or both limits if they are set. \
+       </p>',
 
     'file-descriptors':
       '<p>File descriptor count and limit, as reported by the operating \
@@ -250,6 +269,11 @@ HELP = {
 
     'handle-exe' : 'In order to monitor the number of file descriptors in use on Windows, RabbitMQ needs the <a href="http://technet.microsoft.com/en-us/sysinternals/bb896655" target="_blank">handle.exe command line tool from Microsoft</a>. Download it and place it in the path (e.g. in C:\Windows).',
 
+    'filter-regex' :
+    'Whether to enable regular expression matching. Both string literals \
+    and regular expressions are matched in a case-insensitive manner.<br/></br/> \
+    (<a href="https://developer.mozilla.org/en/docs/Web/JavaScript/Guide/Regular_Expressions" target="_blank">Regular expression reference</a>)',
+
     'foo': 'foo' // No comma.
 };
 
