@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is GoPivotal, Inc.
-%%   Copyright (c) 2010-2013 GoPivotal, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_db).
@@ -137,9 +137,10 @@
           %% {queue/channel/connection}
           old_stats,
           gc_timer,
-          gc_continuation,
+          gc_next_key,
           lookups,
-          interval}).
+          interval,
+          event_refresh_ref}).
 
 -define(FINE_STATS_TYPES, [channel_queue_stats, channel_exchange_stats,
                            channel_queue_exchange_stats]).
@@ -179,9 +180,10 @@ prioritise_cast(_Msg, _Len, _State) ->
 %%----------------------------------------------------------------------------
 
 start_link() ->
-    case gen_server2:start_link({global, ?MODULE}, ?MODULE, [], []) of
+    Ref = make_ref(),
+    case gen_server2:start_link({global, ?MODULE}, ?MODULE, [Ref], []) of
         {ok, Pid} -> register(?MODULE, Pid), %% [1]
-                     rabbit:force_event_refresh(),
+                     rabbit:force_event_refresh(Ref),
                      {ok, Pid};
         Else      -> Else
     end.
@@ -225,7 +227,7 @@ safe_call(Term, Default, Retries) ->
 %% Internal, gen_server2 callbacks
 %%----------------------------------------------------------------------------
 
-init([]) ->
+init([Ref]) ->
     %% When Rabbit is overloaded, it's usually especially important
     %% that the management plugin work.
     process_flag(priority, high),
@@ -240,7 +242,8 @@ init([]) ->
                     tables                 = Tables,
                     old_stats              = Table(),
                     aggregated_stats       = Table(),
-                    aggregated_stats_index = Table()})), hibernate,
+                    aggregated_stats_index = Table(),
+                    event_refresh_ref      = Ref})), hibernate,
      {backoff, ?HIBERNATE_AFTER_MIN, ?HIBERNATE_AFTER_MIN, ?DESIRED_HIBERNATE}}.
 
 handle_call({augment_exchanges, Xs, Ranges, basic}, _From, State) ->
@@ -330,7 +333,14 @@ handle_call(reset_lookups, _From, State) ->
 handle_call(_Request, _From, State) ->
     reply(not_understood, State).
 
-handle_cast({event, Event}, State) ->
+%% Only handle events that are real, or pertain to a force-refresh
+%% that we instigated.
+handle_cast({event, Event = #event{reference = none}}, State) ->
+    handle_event(Event, State),
+    noreply(State);
+
+handle_cast({event, Event = #event{reference = Ref}},
+            State = #state{event_refresh_ref = Ref}) ->
     handle_event(Event, State),
     noreply(State);
 
@@ -438,7 +448,7 @@ handle_event(#event{type = queue_stats, props = Stats, timestamp = Timestamp},
     handle_stats(queue_stats, Stats, Timestamp,
                  [{fun rabbit_mgmt_format:properties/1,[backing_queue_status]},
                   {fun rabbit_mgmt_format:timestamp/1, [idle_since]},
-                  {fun rabbit_mgmt_format:queue_status/1, [status]}],
+                  {fun rabbit_mgmt_format:queue_state/1, [state]}],
                  [messages, messages_ready, messages_unacknowledged], State);
 
 handle_event(Event = #event{type = queue_deleted,
@@ -1032,25 +1042,29 @@ augment_connection_pid(Pid, #state{tables = Tables}) ->
 %% Internal, event-GCing
 %%----------------------------------------------------------------------------
 
-gc_batch(State = #state{aggregated_stats = ETS,
-                        gc_continuation = Continuation}) ->
-    Match = case Continuation of
-                undefined -> Size = ets:info(ETS, size),
-                             Rows = lists:max([?GC_MIN_ROWS,
-                                               round(?GC_MIN_RATIO * Size)]),
-                             ets:match(ETS, '$1', Rows);
-                _         -> ets:match(Continuation)
-            end,
-    case Match of
-        {Matches, Continuation1} ->
-            {ok, Policies} = application:get_env(
-                               rabbitmq_management, sample_retention_policies),
-            Now = floor(erlang:now(), State),
-            [gc(Key, Stats, Policies, Now, ETS) || [{Key, Stats}] <- Matches],
-            State#state{gc_continuation = Continuation1};
-        '$end_of_table' ->
-            State#state{gc_continuation = undefined}
-    end.
+gc_batch(State = #state{aggregated_stats = ETS}) ->
+    {ok, Policies} = application:get_env(
+                       rabbitmq_management, sample_retention_policies),
+    Rows = erlang:max(?GC_MIN_ROWS,
+                      round(?GC_MIN_RATIO * ets:info(ETS, size))),
+    gc_batch(Rows, Policies, State).
+
+gc_batch(0, _Policies, State) ->
+    State;
+gc_batch(Rows, Policies, State = #state{aggregated_stats = ETS,
+                         gc_next_key      = Key0}) ->
+    Key = case Key0 of
+              undefined -> ets:first(ETS);
+              _         -> ets:next(ETS, Key0)
+          end,
+    Key1 = case Key of
+               '$end_of_table' -> undefined;
+               _               -> Now = floor(erlang:now(), State),
+                                  Stats = ets:lookup_element(ETS, Key, 2),
+                                  gc(Key, Stats, Policies, Now, ETS),
+                                  Key
+           end,
+    gc_batch(Rows - 1, Policies, State#state{gc_next_key = Key1}).
 
 gc({{Type, Id}, Key}, Stats, Policies, Now, ETS) ->
     Policy = pget(retention_policy(Type), Policies),
