@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mqtt_frame).
@@ -22,7 +22,6 @@
 -include("rabbit_mqtt_frame.hrl").
 
 -define(RESERVED, 0).
--define(PROTOCOL_MAGIC, "MQIsdp").
 -define(MAX_LEN, 16#fffffff).
 -define(HIGHBIT, 2#10000000).
 -define(LOWBITS, 2#01111111).
@@ -57,7 +56,7 @@ parse_frame(Bin, #mqtt_frame_fixed{ type = Type,
                                     qos  = Qos } = Fixed, Length) ->
     case {Type, Bin} of
         {?CONNECT, <<FrameBin:Length/binary, Rest/binary>>} ->
-            {ProtocolMagic, Rest1} = parse_utf(FrameBin),
+            {ProtoName, Rest1} = parse_utf(FrameBin),
             <<ProtoVersion : 8, Rest2/binary>> = Rest1,
             <<UsernameFlag : 1,
               PasswordFlag : 1,
@@ -73,7 +72,7 @@ parse_frame(Bin, #mqtt_frame_fixed{ type = Type,
             {WillMsg,   Rest6} = parse_msg(Rest5, WillFlag),
             {UserName,  Rest7} = parse_utf(Rest6, UsernameFlag),
             {PasssWord, <<>>}  = parse_utf(Rest7, PasswordFlag),
-            case ProtocolMagic == ?PROTOCOL_MAGIC of
+            case protocol_name_approved(ProtoVersion, ProtoName) of
                 true ->
                     wrap(Fixed,
                          #mqtt_frame_connect{
@@ -228,3 +227,6 @@ opt(undefined)            -> ?RESERVED;
 opt(false)                -> 0;
 opt(true)                 -> 1;
 opt(X) when is_integer(X) -> X.
+
+protocol_name_approved(Ver, Name) ->
+    lists:member({Ver, Name}, ?PROTOCOL_NAMES).
