@@ -11,13 +11,13 @@
 %%  The Original Code is RabbitMQ.
 %%
 %%  The Initial Developer of the Original Code is GoPivotal, Inc.
-%%  Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%%  Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(rabbit_shovel_worker).
 -behaviour(gen_server2).
 
--export([start_link/2]).
+-export([start_link/3]).
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2,
          code_change/3]).
 
@@ -27,46 +27,47 @@
 -define(MAX_CONNECTION_CLOSE_TIMEOUT, 10000).
 
 -record(state, {inbound_conn, inbound_ch, outbound_conn, outbound_ch,
-                name, config, blocked, msg_buf, inbound_params,
-                outbound_params, unacked}).
+                name, type, config, inbound_uri, outbound_uri, unacked,
+                remaining, %% [1]
+                remaining_unacked}). %% [2]
 
-start_link(Name, Config) ->
-    ok = rabbit_shovel_status:report(Name, starting),
-    gen_server2:start_link(?MODULE, [Name, Config], []).
+%% [1] Counts down until we shut down in all modes
+%% [2] Counts down until we stop publishing in on-confirm mode
+
+start_link(Type, Name, Config) ->
+    ok = rabbit_shovel_status:report(Name, Type, starting),
+    gen_server2:start_link(?MODULE, [Type, Name, Config], []).
 
 %%---------------------------
 %% Gen Server Implementation
 %%---------------------------
 
-init([Name, Config]) ->
+init([Type, Name, Config]) ->
     gen_server2:cast(self(), init),
-    {ok, Shovel} = rabbit_shovel_config:parse(Name, Config),
-    {ok, #state{name = Name, config = Shovel}}.
+    {ok, Shovel} = parse(Type, Name, Config),
+    {ok, #state{name = Name, type = Type, config = Shovel}}.
+
+parse(static,  Name, Config) -> rabbit_shovel_config:parse(Name, Config);
+parse(dynamic, Name, Config) -> rabbit_shovel_parameters:parse(Name, Config).
 
 handle_call(_Msg, _From, State) ->
     {noreply, State}.
 
 handle_cast(init, State = #state{config = Config}) ->
-    %% TODO when we move to minimum R13B01:
-    %% random:seed(now()),
-    {A, B, C} = now(),
-    random:seed(A, B, C),
+    random:seed(now()),
     #shovel{sources = Sources, destinations = Destinations} = Config,
-    {InboundConn, InboundChan, InboundParams} =
-        make_conn_and_chan(Sources#endpoint.amqp_params),
-    {OutboundConn, OutboundChan, OutboundParams} =
-        make_conn_and_chan(Destinations#endpoint.amqp_params),
+    {InboundConn, InboundChan, InboundURI} =
+        make_conn_and_chan(Sources#endpoint.uris),
+    {OutboundConn, OutboundChan, OutboundURI} =
+        make_conn_and_chan(Destinations#endpoint.uris),
 
     %% Don't trap exits until we have established connections so that
     %% if we try to shut down while waiting for a connection to be
     %% established then we don't block
     process_flag(trap_exit, true),
 
-    create_resources(InboundChan,
-                     Sources#endpoint.resource_declarations),
-
-    create_resources(OutboundChan,
-                     Destinations#endpoint.resource_declarations),
+    (Sources#endpoint.resource_declaration)(InboundConn, InboundChan),
+    (Destinations#endpoint.resource_declaration)(OutboundConn, OutboundChan),
 
     NoAck = Config#shovel.ack_mode =:= no_ack,
     case NoAck of
@@ -77,8 +78,6 @@ handle_cast(init, State = #state{config = Config}) ->
         true  -> ok
     end,
 
-    ok = amqp_channel:register_flow_handler(OutboundChan, self()),
-
     case Config#shovel.ack_mode of
         on_confirm ->
             #'confirm.select_ok'{} =
@@ -88,6 +87,8 @@ handle_cast(init, State = #state{config = Config}) ->
             ok
     end,
 
+    Remaining = remaining(InboundChan, Config),
+
     #'basic.consume_ok'{} =
         amqp_channel:subscribe(
           InboundChan, #'basic.consume'{queue  = Config#shovel.queue,
@@ -97,11 +98,12 @@ handle_cast(init, State = #state{config = Config}) ->
     State1 =
         State#state{inbound_conn = InboundConn, inbound_ch = InboundChan,
                     outbound_conn = OutboundConn, outbound_ch = OutboundChan,
-                    blocked = false, msg_buf = queue:new(),
-                    inbound_params = InboundParams,
-                    outbound_params = OutboundParams,
+                    inbound_uri = InboundURI,
+                    outbound_uri = OutboundURI,
+                    remaining = Remaining,
+                    remaining_unacked = Remaining,
                     unacked = gb_trees:empty()},
-    ok = report_status(running, State1),
+    ok = report_running(State1),
     {noreply, State1}.
 
 handle_info(#'basic.consume_ok'{}, State) ->
@@ -110,40 +112,27 @@ handle_info(#'basic.consume_ok'{}, State) ->
 handle_info({#'basic.deliver'{delivery_tag = Tag,
                               exchange = Exchange, routing_key = RoutingKey},
              Msg = #amqp_msg{props = Props = #'P_basic'{}}},
-            State = #state{config = Config}) ->
+            State = #state{inbound_uri  = InboundURI,
+                           outbound_uri = OutboundURI,
+                           config = #shovel{publish_properties = PropsFun,
+                                            publish_fields     = FieldsFun}}) ->
     Method = #'basic.publish'{exchange = Exchange, routing_key = RoutingKey},
-    Method1 = (Config#shovel.publish_fields)(Method),
-    Msg1 = Msg#amqp_msg{props = (Config#shovel.publish_properties)(Props)},
+    Method1 = FieldsFun(InboundURI, OutboundURI, Method),
+    Msg1 = Msg#amqp_msg{props = PropsFun(InboundURI, OutboundURI, Props)},
     {noreply, publish(Tag, Method1, Msg1, State)};
 
-handle_info(#'channel.flow'{active = true},
-            State = #state{inbound_ch = InboundChan}) ->
-    ok = report_status(running, State),
-    State1 = drain_buffer(State#state{blocked = false}),
-    ok = case State1#state.blocked of
-             true  -> report_status(blocked, State);
-             false -> channel_flow(InboundChan, true)
-         end,
-    {noreply, State1};
-
-handle_info(#'channel.flow'{active = false},
-            State = #state{inbound_ch = InboundChan}) ->
-    ok = report_status(blocked, State),
-    ok = channel_flow(InboundChan, false),
-    {noreply, State#state{blocked = true}};
-
 handle_info(#'basic.ack'{delivery_tag = Seq, multiple = Multiple},
             State = #state{config = #shovel{ack_mode = on_confirm}}) ->
     {noreply, confirm_to_inbound(
-                fun (DTag, Multiple) ->
-                        #'basic.ack'{delivery_tag = DTag, multiple = Multiple}
+                fun (DTag, Multi) ->
+                        #'basic.ack'{delivery_tag = DTag, multiple = Multi}
                 end, Seq, Multiple, State)};
 
 handle_info(#'basic.nack'{delivery_tag = Seq, multiple = Multiple},
             State = #state{config = #shovel{ack_mode = on_confirm}}) ->
     {noreply, confirm_to_inbound(
-                fun (DTag, Multiple) ->
-                        #'basic.nack'{delivery_tag = DTag, multiple = Multiple}
+                fun (DTag, Multi) ->
+                        #'basic.nack'{delivery_tag = DTag, multiple = Multi}
                 end, Seq, Multiple, State)};
 
 handle_info(#'basic.cancel'{}, State = #state{name = Name}) ->
@@ -161,15 +150,17 @@ handle_info({'EXIT', OutboundConn, Reason},
 
 terminate(Reason, #state{inbound_conn = undefined, inbound_ch = undefined,
                          outbound_conn = undefined, outbound_ch = undefined,
-                         name = Name}) ->
-    rabbit_shovel_status:report(Name, {terminated, Reason}),
+                         name = Name, type = Type}) ->
+    rabbit_shovel_status:report(Name, Type, {terminated, Reason}),
     ok;
 terminate(Reason, State) ->
+    maybe_autodelete(Reason, State),
     catch amqp_connection:close(State#state.inbound_conn,
                                 ?MAX_CONNECTION_CLOSE_TIMEOUT),
     catch amqp_connection:close(State#state.outbound_conn,
                                 ?MAX_CONNECTION_CLOSE_TIMEOUT),
-    rabbit_shovel_status:report(State#state.name, {terminated, Reason}),
+    rabbit_shovel_status:report(State#state.name, State#state.type,
+                                {terminated, Reason}),
     ok.
 
 code_change(_OldVsn, State, _Extra) ->
@@ -183,77 +174,89 @@ confirm_to_inbound(MsgCtr, Seq, Multiple, State =
                        #state{inbound_ch = InboundChan, unacked = Unacked}) ->
     ok = amqp_channel:cast(
            InboundChan, MsgCtr(gb_trees:get(Seq, Unacked), Multiple)),
-    Unacked1 = remove_delivery_tags(Seq, Multiple, Unacked),
-    State#state{unacked = Unacked1}.
+    {Unacked1, Removed} = remove_delivery_tags(Seq, Multiple, Unacked, 0),
+    decr_remaining(Removed, State#state{unacked = Unacked1}).
 
-remove_delivery_tags(Seq, false, Unacked) ->
-    gb_trees:delete(Seq, Unacked);
-remove_delivery_tags(Seq, true, Unacked) ->
+remove_delivery_tags(Seq, false, Unacked, 0) ->
+    {gb_trees:delete(Seq, Unacked), 1};
+remove_delivery_tags(Seq, true, Unacked, Count) ->
     case gb_trees:is_empty(Unacked) of
-        true  -> Unacked;
+        true  -> {Unacked, Count};
         false -> {Smallest, _Val, Unacked1} = gb_trees:take_smallest(Unacked),
                  case Smallest > Seq of
-                     true  -> Unacked;
-                     false -> remove_delivery_tags(Seq, true, Unacked1)
+                     true  -> {Unacked, Count};
+                     false -> remove_delivery_tags(Seq, true, Unacked1, Count+1)
                  end
     end.
 
-report_status(Verb, State) ->
+report_running(State) ->
     rabbit_shovel_status:report(
-      State#state.name, {Verb, {source, State#state.inbound_params},
-                         {destination, State#state.outbound_params}}).
+      State#state.name, State#state.type,
+      {running, [{src_uri,  State#state.inbound_uri},
+                 {dest_uri, State#state.outbound_uri}]}).
+
+publish(_Tag, _Method, _Msg, State = #state{remaining_unacked = 0}) ->
+    %% We are in on-confirm mode, and are autodelete. We have
+    %% published all the messages we need to; we just wait for acks to
+    %% come back. So drop subsequent messages on the floor to be
+    %% requeued later.
+    State;
 
 publish(Tag, Method, Msg,
         State = #state{inbound_ch = InboundChan, outbound_ch = OutboundChan,
-                       config = Config, blocked = false, msg_buf = MsgBuf,
-                       unacked = Unacked}) ->
+                       config = Config, unacked = Unacked}) ->
     Seq = case Config#shovel.ack_mode of
               on_confirm  -> amqp_channel:next_publish_seqno(OutboundChan);
               _           -> undefined
           end,
-    case amqp_channel:call(OutboundChan, Method, Msg) of
-        ok ->
-            case Config#shovel.ack_mode of
-                no_ack ->
-                    State;
-                on_confirm ->
-                    State#state{unacked = gb_trees:insert(Seq, Tag, Unacked)};
-                on_publish ->
-                    ok = amqp_channel:cast(
-                           InboundChan, #'basic.ack'{delivery_tag = Tag}),
-                    State
-            end;
-        blocked ->
-            ok = report_status(blocked, State),
-            ok = channel_flow(InboundChan, false),
-            State#state{blocked = true,
-                        msg_buf = queue:in_r({Tag, Method, Msg}, MsgBuf)}
-    end;
-publish(Tag, Method, Msg, State = #state{blocked = true, msg_buf = MsgBuf}) ->
-    State#state{msg_buf = queue:in({Tag, Method, Msg}, MsgBuf)}.
+    ok = amqp_channel:call(OutboundChan, Method, Msg),
+    decr_remaining_unacked(
+      case Config#shovel.ack_mode of
+          no_ack     -> decr_remaining(1, State);
+          on_confirm -> State#state{unacked = gb_trees:insert(
+                                                Seq, Tag, Unacked)};
+          on_publish -> ok = amqp_channel:cast(
+                               InboundChan, #'basic.ack'{delivery_tag = Tag}),
+                        decr_remaining(1, State)
+      end).
 
-drain_buffer(State = #state{blocked = true}) ->
-    State;
-drain_buffer(State = #state{blocked = false, msg_buf = MsgBuf}) ->
-    case queue:out(MsgBuf) of
-        {empty, _MsgBuf} ->
-            State;
-        {{value, {Tag, Method, Msg}}, MsgBuf1} ->
-            drain_buffer(publish(Tag, Method, Msg,
-                                 State#state{msg_buf = MsgBuf1}))
-    end.
-
-channel_flow(Chan, Active) ->
-    #'channel.flow_ok'{active = Active} =
-        amqp_channel:call(Chan, #'channel.flow'{active = Active}),
-    ok.
-
-make_conn_and_chan(AmqpParams) ->
-    AmqpParam = lists:nth(random:uniform(length(AmqpParams)), AmqpParams),
+make_conn_and_chan(URIs) ->
+    URI = lists:nth(random:uniform(length(URIs)), URIs),
+    {ok, AmqpParam} = amqp_uri:parse(URI),
     {ok, Conn} = amqp_connection:start(AmqpParam),
     link(Conn),
     {ok, Chan} = amqp_connection:open_channel(Conn),
-    {Conn, Chan, AmqpParam}.
+    {Conn, Chan, list_to_binary(amqp_uri:remove_credentials(URI))}.
 
-create_resources(Chan, Declarations) ->
-    [amqp_channel:call(Chan, Method) || Method <- Declarations].
+remaining(Ch, #shovel{delete_after = never}) ->
+    unlimited;
+remaining(Ch, #shovel{delete_after = 'queue-length', queue = Queue}) ->
+    #'queue.declare_ok'{message_count = N} =
+        amqp_channel:call(Ch, #'queue.declare'{queue   = Queue,
+                                               passive = true}),
+    N;
+remaining(Ch, #shovel{delete_after = Count}) ->
+    Count.
+
+decr_remaining(_N, State = #state{remaining = unlimited}) ->
+    State;
+decr_remaining(N,  State = #state{remaining = M}) ->
+    case M > N of
+        true  -> State#state{remaining = M - N};
+        false -> exit({shutdown, autodelete})
+    end.
+
+decr_remaining_unacked(State = #state{remaining_unacked = unlimited}) ->
+    State;
+decr_remaining_unacked(State = #state{remaining_unacked = 0}) ->
+    State;
+decr_remaining_unacked(State = #state{remaining_unacked = N}) ->
+    State#state{remaining_unacked = N - 1}.
+
+maybe_autodelete({shutdown, autodelete}, #state{name = {VHost, Name},
+                                                type = dynamic}) ->
+    %% See rabbit_shovel_dyn_worker_sup_sup:stop_child/1
+    put(shovel_worker_autodelete, true),
+    rabbit_runtime_parameters:clear(VHost, <<"shovel">>, Name);
+maybe_autodelete(_Reason, _State) ->
+    ok.
