@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(rabbit_stomp_reader).
@@ -41,40 +41,48 @@ init(SupHelperPid, ProcessorPid, Configuration) ->
     Reply.
 
 go(SupHelperPid, ProcessorPid, Configuration) ->
+    process_flag(trap_exit, true),
     receive
         {go, Sock0, SockTransform} ->
-            {ok, Sock} = SockTransform(Sock0),
-            case rabbit_net:connection_string(Sock, inbound) of
+            case rabbit_net:connection_string(Sock0, inbound) of
                 {ok, ConnStr} ->
-                    ProcInitArgs = processor_args(SupHelperPid, Configuration, Sock),
-                    rabbit_stomp_processor:init_arg(ProcessorPid, ProcInitArgs),
-                    log(info, "accepting STOMP connection ~p (~s)~n",
-                        [self(), ConnStr]),
+                    case SockTransform(Sock0) of
+                        {ok, Sock} ->
 
-                    ParseState = rabbit_stomp_frame:initial_state(),
-                    try
-                        mainloop(
-                          register_resource_alarm(
-                            #reader_state{socket             = Sock,
-                                          parse_state        = ParseState,
-                                          processor          = ProcessorPid,
-                                          state              = running,
-                                          conserve_resources = false,
-                                          recv_outstanding   = false})),
-                        log(info, "closing STOMP connection ~p (~s)~n",
-                            [self(), ConnStr])
-                    catch
-                        _:Ex -> log(error, "closing STOMP connection "
-                                    "~p (~s):~n~p~n", [self(), ConnStr, Ex])
-                    end,
-                    done;
-                {error, enotconn} ->
-                    rabbit_net:fast_close(Sock),
-                    done;
-                {error, Reason} ->
-                    log(warning, "STOMP network error while starting up: ~p~n",
-                        [Reason]),
-                    rabbit_net:fast_close(Sock)
+                            ProcInitArgs = processor_args(SupHelperPid,
+                                                          Configuration,
+                                                          Sock),
+                            rabbit_stomp_processor:init_arg(ProcessorPid,
+                                                            ProcInitArgs),
+                            log(info, "accepting STOMP connection ~p (~s)~n",
+                                [self(), ConnStr]),
+
+                            ParseState = rabbit_stomp_frame:initial_state(),
+                            try
+                                mainloop(
+                                  register_resource_alarm(
+                                    #reader_state{socket             = Sock,
+                                                  parse_state        = ParseState,
+                                                  processor          = ProcessorPid,
+                                                  state              = running,
+                                                  conserve_resources = false,
+                                                  recv_outstanding   = false})),
+                                log(info, "closing STOMP connection ~p (~s)~n",
+                                    [self(), ConnStr])
+                            catch _:Ex ->
+                                log_network_error(ConnStr, Ex),
+                                rabbit_net:fast_close(Sock),
+                                exit(normal)
+                            end,
+                            done;
+                        {error, enotconn} ->
+                            rabbit_net:fast_close(Sock0),
+                            exit(normal);
+                        {error, Reason} ->
+                            log_network_error(ConnStr, Reason),
+                            rabbit_net:fast_close(Sock0),
+                            exit(normal)
+                        end
             end
     end.
 
@@ -92,7 +100,13 @@ mainloop(State0 = #reader_state{socket = Sock}) ->
             mainloop(State#reader_state{conserve_resources = Conserve});
         {bump_credit, Msg} ->
             credit_flow:handle_bump_msg(Msg),
-            mainloop(State)
+            mainloop(State);
+        {'EXIT', _From, shutdown} ->
+            ok;
+        Other ->
+            log(warning, "STOMP connection ~p received "
+                "an unexpected message ~p~n", [Other]),
+            ok
     end.
 
 process_received_bytes([], State) ->
@@ -162,8 +176,9 @@ processor_args(SupPid, Configuration, Sock) ->
                 rabbit_heartbeat:start(SupPid, Sock, SendTimeout,
                                        SendFin, ReceiveTimeout, ReceiveFun)
         end,
+    {ok, {PeerAddr, _PeerPort}} = rabbit_net:sockname(Sock),
     [SendFun, adapter_info(Sock), StartHeartbeatFun,
-     ssl_login_name(Sock, Configuration)].
+     ssl_login_name(Sock, Configuration), PeerAddr].
 
 adapter_info(Sock) ->
     amqp_connection:socket_adapter_info(Sock, {'STOMP', 0}).
@@ -180,3 +195,28 @@ ssl_login_name(Sock, #stomp_configuration{ssl_cert_login = true}) ->
         {error, no_peercert} -> none;
         nossl                -> none
     end.
+
+%%----------------------------------------------------------------------------
+
+log_network_error(ConnStr, {ssl_upgrade_error,
+                            {tls_alert, "handshake failure"}}) ->
+    log(error, "STOMP detected TLS upgrade error on "
+        "~p (~s): handshake failure~n", [self(), ConnStr]);
+
+log_network_error(ConnStr, {ssl_upgrade_error,
+                            {tls_alert, "unknown ca"}}) ->
+    log(error, "STOMP detected TLS certificate "
+        "verification error on "
+        "~p (~s): alert 'unknown CA'~n", [self(), ConnStr]);
+
+log_network_error(ConnStr, {ssl_upgrade_error, {tls_alert, Alert}}) ->
+    log(error, "STOMP detected TLS upgrade error on "
+        "~p (~s): alert ~s~n", [self(), ConnStr, Alert]);
+
+log_network_error(ConnStr, {ssl_upgrade_error, closed}) ->
+    log(error, "STOMP detected TLS upgrade error on "
+        "~p (~s): connection closed~n", [self(), ConnStr]);
+
+log_network_error(ConnStr, Ex) ->
+    log(error, "STOMP detected network error on "
+        "~p (~s):~n~p~n", [self(), ConnStr, Ex]).
