@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2011-2013 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2011-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(mirrored_supervisor).
@@ -446,10 +446,10 @@ supervisor(Pid) -> with_exit_handler(fun() -> dead end,
                                      fun() -> delegate(Pid) end).
 
 write(Group, Overall, ChildSpec) ->
-    ok = mnesia:write(
-           #mirrored_sup_childspec{key           = {Group, id(ChildSpec)},
-                                   mirroring_pid = Overall,
-                                   childspec     = ChildSpec}),
+    S = #mirrored_sup_childspec{key           = {Group, id(ChildSpec)},
+                                mirroring_pid = Overall,
+                                childspec     = ChildSpec},
+    ok = mnesia:write(?TABLE, S, write),
     ChildSpec.
 
 delete(Group, Id) ->
