@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(rabbit_sup).
@@ -21,7 +21,9 @@
 -export([start_link/0, start_child/1, start_child/2, start_child/3,
          start_supervisor_child/1, start_supervisor_child/2,
          start_supervisor_child/3,
-         start_restartable_child/1, start_restartable_child/2, stop_child/1]).
+         start_restartable_child/1, start_restartable_child/2,
+         start_delayed_restartable_child/1, start_delayed_restartable_child/2,
+         stop_child/1]).
 
 -export([init/1]).
 
@@ -42,6 +44,8 @@
 -spec(start_supervisor_child/3 :: (atom(), atom(), [any()]) -> 'ok').
 -spec(start_restartable_child/1 :: (atom()) -> 'ok').
 -spec(start_restartable_child/2 :: (atom(), [any()]) -> 'ok').
+-spec(start_delayed_restartable_child/1 :: (atom()) -> 'ok').
+-spec(start_delayed_restartable_child/2 :: (atom(), [any()]) -> 'ok').
 -spec(stop_child/1 :: (atom()) -> rabbit_types:ok_or_error(any())).
 
 -endif.
@@ -70,14 +74,17 @@ start_supervisor_child(ChildId, Mod, Args) ->
                   {ChildId, {Mod, start_link, Args},
                    transient, infinity, supervisor, [Mod]})).
 
-start_restartable_child(Mod) -> start_restartable_child(Mod, []).
+start_restartable_child(M)            -> start_restartable_child(M, [], false).
+start_restartable_child(M, A)         -> start_restartable_child(M, A,  false).
+start_delayed_restartable_child(M)    -> start_restartable_child(M, [], true).
+start_delayed_restartable_child(M, A) -> start_restartable_child(M, A,  true).
 
-start_restartable_child(Mod, Args) ->
+start_restartable_child(Mod, Args, Delay) ->
     Name = list_to_atom(atom_to_list(Mod) ++ "_sup"),
     child_reply(supervisor:start_child(
                   ?SERVER,
                   {Name, {rabbit_restartable_sup, start_link,
-                          [Name, {Mod, start_link, Args}]},
+                          [Name, {Mod, start_link, Args}, Delay]},
                    transient, infinity, supervisor, [rabbit_restartable_sup]})).
 
 stop_child(ChildId) ->
