@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2013 GoPivotal, Inc.  All rights reserved.
+%% Copyright (c) 2007-2014 GoPivotal, Inc.  All rights reserved.
 %%
 
 -module(tcp_acceptor).
@@ -63,9 +63,9 @@ handle_info({inet_async, LSock, Ref, {ok, Sock}},
         {error, Err}      -> {ok, {IPAddress, Port}} = inet:sockname(LSock),
                              error_logger:error_msg(
                                "failed to tune buffer size of "
-                               "connection accepted on ~s:~p - ~p (~s)~n",
+                               "connection accepted on ~s:~p - ~s~n",
                                [rabbit_misc:ntoab(IPAddress), Port,
-                                Err, rabbit_misc:format_inet_error(Err)]),
+                                rabbit_misc:format_inet_error(Err)]),
                              catch port_close(Sock)
     end,
 
